@@ -3,7 +3,7 @@
 <description>Generate artificial data sets with a simple 'Paint' like interface</description>
 <contact>Ales Erjavec (ales.erjavec(@at@)fri.uni-lj.si)</contact>
 <priority>40</priority>
-<icon>icons/PaintData.png</icon>
+<icon>icons/PaintData.svg</icon>
 """
 
 import orange
@@ -690,7 +690,7 @@ class OWPaintData(OWWidget):
              ]
     settingsList = ["commitOnChange"]
     def __init__(self, parent=None, signalManager=None, name="Data Generator"):
-        OWWidget.__init__(self, parent, signalManager, name, wantGraph=True)
+        OWWidget.__init__(self, parent, signalManager, name)
         
         self.outputs = [("Data", ExampleTable)]
         
