@@ -1,7 +1,7 @@
 """
 <name>Test Learners</name>
 <description>Estimates the predictive performance of learners on a data set.</description>
-<icon>icons/TestLearners.png</icon>
+<icon>icons/TestLearners1.svg</icon>
 <contact>Blaz Zupan (blaz.zupan(@at@)fri.uni-lj.si)</contact>
 <priority>200</priority>
 """
