@@ -1,13 +1,16 @@
 """
 <name>k Nearest Neighbours Regression</name>
 <description>K-nearest neighbours learner/predictor.</description>
-<icon>icons/kNearestNeighbours.png</icon>
+<icon>icons/kNearestNeighbours.svg</icon>
 <contact>Janez Demsar (janez.demsar(@at@)fri.uni-lj.si)</contact>
 <priority>20</priority>
 <keywords>knn</keywords>
 """
+from OWWidget import *
+
+from OWKNN import OWKNN
+from orngWrap import PreprocessedLearner
 
-from OWKNN import *
 
 class OWKNNRegression(OWKNN):
     def __init__(self, parent=None, signalManager=None, title="kNN Regression"):
