@@ -1,6 +1,7 @@
 import Orange
 from Orange.utils import deprecated_keywords, deprecated_members
 from Orange.data import preprocess
+from Orange.data.continuization import DomainContinuizer
 import decimal
 import math
 
@@ -104,7 +105,7 @@ class LogRegLearner(Orange.classification.Learner):
     @deprecated_keywords({"weightID": "weight_id"})
     def __new__(cls, data=None, weight_id=0, **argkw):
         self = Orange.classification.Learner.__new__(cls, **argkw)
-        if data:
+        if data is not None:
             self.__init__(**argkw)
             return self.__call__(data, weight_id)
         else:
@@ -181,7 +182,7 @@ LogRegLearner = deprecated_members({"removeSingular": "remove_singular",
 class UnivariateLogRegLearner(Orange.classification.Learner):
     def __new__(cls, data=None, **argkw):
         self = Orange.classification.Learner.__new__(cls, **argkw)
-        if data:
+        if data is not None:
             self.__init__(**argkw)
             return self.__call__(data)
         else:
@@ -219,7 +220,7 @@ class UnivariateLogRegClassifier(Orange.classification.Classifier):
 class LogRegLearnerGetPriors(object):
     def __new__(cls, data=None, weight_id=0, **argkw):
         self = object.__new__(cls)
-        if data:
+        if data is not None:
             self.__init__(**argkw)
             return self.__call__(data, weight_id)
         else:
@@ -891,7 +892,7 @@ StepWiseFSS = deprecated_members({"addCrit": "add_crit",
 class StepWiseFSSFilter(object):
     def __new__(cls, data=None, **argkw):
         self = object.__new__(cls)
-        if data:
+        if data is not None:
             self.__init__(**argkw)
             return self.__call__(data)
         else:
@@ -919,29 +920,30 @@ StepWiseFSSFilter = deprecated_members({"addCrit": "add_crit",
 ####################################
 ##  PROBABILITY CALCULATIONS
 
-def lchisqprob(chisq,df):
+def lchisqprob(chisq, df):
     """
     Return the (1-tailed) probability value associated with the provided
     chi-square value and df.  Adapted from chisq.c in Gary Perlman's |Stat.
     """
     BIG = 20.0
+
     def ex(x):
-    	BIG = 20.0
-    	if x < -BIG:
-    	    return 0.0
-    	else:
-    	    return math.exp(x)
-    if chisq <=0 or df < 1:
-    	return 1.0
+        BIG = 20.0
+        if x < -BIG:
+            return 0.0
+        else:
+            return math.exp(x)
+    if chisq <= 0 or df < 1:
+        return 1.0
     a = 0.5 * chisq
-    if df%2 == 0:
-    	even = 1
+    if df % 2 == 0:
+        even = 1
     else:
-    	even = 0
+        even = 0
     if df > 1:
-    	y = ex(-a)
+        y = ex(-a)
     if even:
-    	s = y
+        s = y
     else:
         s = 2.0 * zprob(-math.sqrt(chisq))
     if (df > 2):
@@ -952,14 +954,14 @@ def lchisqprob(chisq,df):
             z = 0.5
         if a > BIG:
             if even:
-            	e = 0.0
+                e = 0.0
             else:
-            	e = math.log(math.sqrt(math.pi))
+                e = math.log(math.sqrt(math.pi))
             c = math.log(a)
             while (z <= chisq):
-            	e = math.log(z) + e
-            	s = s + ex(c*z-a-e)
-            	z = z + 1.0
+                e = math.log(z) + e
+                s = s + ex(c * z - a - e)
+                z = z + 1.0
             return s
         else:
             if even:
@@ -968,10 +970,10 @@ def lchisqprob(chisq,df):
                 e = 1.0 / math.sqrt(math.pi) / math.sqrt(a)
             c = 0.0
             while (z <= chisq):
-                e = e * (a/float(z))
+                e = e * (a / float(z))
                 c = c + e
                 z = z + 1.0
-            return (c*y+s)
+            return (c * y + s)
     else:
         return s
 
@@ -979,7 +981,7 @@ def lchisqprob(chisq,df):
 def zprob(z):
     """
     Returns the area under the normal curve 'to the left of' the given z value.
-    Thus:: 
+    Thus::
 
     for z<0, zprob(z) = 1-tail probability
     for z>0, 1.0-zprob(z) = 1-tail probability
@@ -989,32 +991,32 @@ def zprob(z):
     """
     Z_MAX = 6.0    # maximum meaningful z-value
     if z == 0.0:
-	x = 0.0
+        x = 0.0
     else:
-	y = 0.5 * math.fabs(z)
-	if y >= (Z_MAX*0.5):
-	    x = 1.0
-	elif (y < 1.0):
-	    w = y*y
-	    x = ((((((((0.000124818987 * w
-			-0.001075204047) * w +0.005198775019) * w
-		      -0.019198292004) * w +0.059054035642) * w
-		    -0.151968751364) * w +0.319152932694) * w
-		  -0.531923007300) * w +0.797884560593) * y * 2.0
-	else:
-	    y = y - 2.0
-	    x = (((((((((((((-0.000045255659 * y
-			     +0.000152529290) * y -0.000019538132) * y
-			   -0.000676904986) * y +0.001390604284) * y
-			 -0.000794620820) * y -0.002034254874) * y
-		       +0.006549791214) * y -0.010557625006) * y
-		     +0.011630447319) * y -0.009279453341) * y
-		   +0.005353579108) * y -0.002141268741) * y
-		 +0.000535310849) * y +0.999936657524
+        y = 0.5 * math.fabs(z)
+    if y >= (Z_MAX * 0.5):
+        x = 1.0
+    elif (y < 1.0):
+        w = y * y
+        x = ((((((((0.000124818987 * w
+            - 0.001075204047) * w + 0.005198775019) * w
+              - 0.019198292004) * w + 0.059054035642) * w
+            - 0.151968751364) * w + 0.319152932694) * w
+          - 0.531923007300) * w + 0.797884560593) * y * 2.0
+    else:
+        y = y - 2.0
+        x = (((((((((((((-0.000045255659 * y
+                 + 0.000152529290) * y - 0.000019538132) * y
+               - 0.000676904986) * y + 0.001390604284) * y
+             - 0.000794620820) * y - 0.002034254874) * y
+               + 0.006549791214) * y - 0.010557625006) * y
+             + 0.011630447319) * y - 0.009279453341) * y
+           + 0.005353579108) * y - 0.002141268741) * y
+         + 0.000535310849) * y + 0.999936657524
     if z > 0.0:
-	prob = ((x+1.0)*0.5)
+        prob = ((x + 1.0) * 0.5)
     else:
-	prob = ((1.0-x)*0.5)
+        prob = ((1.0 - x) * 0.5)
     return prob
 
 
@@ -1022,14 +1024,14 @@ def zprob(z):
 Logistic regression learner from LIBLINEAR
 """
 
-from Orange.data import preprocess
+
 class LibLinearLogRegLearner(Orange.core.LinearLearner):
     """A logistic regression learner from `LIBLINEAR`_.
-    
+
     Supports L2 regularized learning.
-    
+
     .. _`LIBLINEAR`: http://www.csie.ntu.edu.tw/~cjlin/liblinear/
-     
+
     """
 
     L2R_LR = Orange.core.LinearLearner.L2R_LR
@@ -1039,31 +1041,44 @@ class LibLinearLogRegLearner(Orange.core.LinearLearner):
     __new__ = Orange.utils._orange__new__(base=Orange.core.LinearLearner)
 
     def __init__(self, solver_type=L2R_LR, C=1.0, eps=0.01, normalization=True,
-            bias=-1.0, **kwargs):
+            bias=-1.0, multinomial_treatment=DomainContinuizer.NValues,
+            **kwargs):
         """
-        :param solver_type: One of the following class constants: 
+        :param solver_type: One of the following class constants:
             ``L2_LR``, ``L2_LR_DUAL``, ``L1R_LR``.
-        
-        :param C: Regularization parameter (default 1.0). Higher values of C mean 
-            less regularization (C is a coefficient for the loss function).
+
+        :param C: Regularization parameter (default 1.0). Higher values of
+            C mean less regularization (C is a coefficient for the loss
+            function).
         :type C: float
-        
+
         :param eps: Stopping criteria (default 0.01)
         :type eps: float
-        
+
         :param normalization: Normalize the input data prior to learning
             (default True)
         :type normalization: bool
 
         :param bias: If positive, use it as a bias (default -1).
         :type bias: float
-        
+
+        :param multinomial_treatment: Defines how to handle multinomial
+            features for learning. It can be one of the
+            :class:`~.DomainContinuizer` `multinomial_treatment`
+            constants (default: `DomainContinuizer.NValues`).
+
+        :type multinomial_treatment: int
+
+        .. versionadded:: 2.6.1
+            Added `multinomial_treatment`
+
         """
         self.solver_type = solver_type
         self.C = C
         self.eps = eps
         self.normalization = normalization
         self.bias = bias
+        self.multinomial_treatment = multinomial_treatment
 
         for name, value in kwargs.items():
             setattr(self, name, value)
@@ -1073,11 +1088,11 @@ class LibLinearLogRegLearner(Orange.core.LinearLearner):
             raise TypeError("Can only learn a discrete class.")
 
         if data.domain.has_discrete_attributes(False) or self.normalization:
-            dc = Orange.data.continuization.DomainContinuizer()
-            dc.multinomial_treatment = dc.NValues
+            dc = DomainContinuizer()
+            dc.multinomial_treatment = self.multinomial_treatment
             dc.class_treatment = dc.Ignore
             dc.continuous_treatment = \
                     dc.NormalizeByVariance if self.normalization else dc.Leave
-            c_domain = dc(data) 
+            c_domain = dc(data)
             data = data.translate(c_domain)
         return super(LibLinearLogRegLearner, self).__call__(data, weight_id)
