@@ -19,6 +19,7 @@ from Orange.core import LinearClassifier, \
                         SVMClassifierSparse as _SVMClassifierSparse
 
 from Orange.data import preprocess
+from Orange.data.preprocess import DomainContinuizer
 
 from Orange import feature as variable
 
@@ -85,7 +86,10 @@ class SVMLearner(_SVMLearner):
     :type probability: bool
     :param shrinking: use shrinking heuristics
     :type shrinking: bool
-    :param normalization: normalize the input data prior to learning
+    :param normalization: normalize the input data prior to learning into
+        range [0..1] and replace discrete features with indicator columns
+        one for each value of the feature using
+        :class:`~Orange.data.continuization.DomainContinuizer` class
         (default ``True``)
     :type normalization: bool
     :param weight: a list of class weights
@@ -801,7 +805,8 @@ class LinearSVMLearner(Orange.core.LinearLearner):
     __new__ = _orange__new__(base=Orange.core.LinearLearner)
 
     def __init__(self, solver_type=L2R_L2LOSS_DUAL, C=1.0, eps=0.01,
-                 bias=1.0, normalization=True, **kwargs):
+                 bias=1.0, normalization=True,
+                 multinomial_treatment=DomainContinuizer.NValues, **kwargs):
         """
         :param solver_type: One of the following class constants:
             ``L2R_L2LOSS_DUAL``, ``L2R_L2LOSS``,
@@ -825,10 +830,26 @@ class LinearSVMLearner(Orange.core.LinearLearner):
 
         :type bias: float
 
-        :param normalization: Normalize the input data prior to learning
-            (default True)
+        :param normalization: Normalize the input data into range [0..1] prior
+            to learning (default ``True``)
         :type normalization: bool
 
+        :param multinomial_treatment: Defines how to handle multinomial
+            features for learning. It can be one of the
+            :class:`~.DomainContinuizer` `multinomial_treatment`
+            constants (default: `DomainContinuizer.NValues`).
+
+        :type multinomial_treatment: int
+
+        .. versionadded:: 2.6.1
+            Added `multinomial_treatment`
+
+        .. note:: By default if the training data contains discrete features
+            they are replaced by indicator columns one for each value of the
+            feature regardless of the value of `normalization`. This is
+            different then in :class:`SVMLearner` where this is done only if
+            `normalization` is ``True``.
+
         Example
 
             >>> linear_svm = LinearSVMLearner(
@@ -842,25 +863,27 @@ class LinearSVMLearner(Orange.core.LinearLearner):
         self.C = C
         self.bias = bias
         self.normalization = normalization
+        self.multinomial_treatment = multinomial_treatment
 
         for name, val in kwargs.items():
             setattr(self, name, val)
+
         if self.solver_type not in [self.L2R_L2LOSS_DUAL, self.L2R_L2LOSS,
                 self.L2R_L1LOSS_DUAL, self.L1R_L2LOSS]:
-            warnings.warn("""\
-Deprecated 'solver_type', use
-'Orange.classification.logreg.LibLinearLogRegLearner'
-to build a logistic regression model using LIBLINEAR.
-""",
-                DeprecationWarning)
+            warnings.warn(
+                " Deprecated 'solver_type', use "
+                "'Orange.classification.logreg.LibLinearLogRegLearner'"
+                "to build a logistic regression models using LIBLINEAR.",
+                DeprecationWarning
+            )
 
     def __call__(self, data, weight_id=None):
         if not isinstance(data.domain.class_var, variable.Discrete):
             raise TypeError("Can only learn a discrete class.")
 
         if data.domain.has_discrete_attributes(False) or self.normalization:
-            dc = Orange.data.continuization.DomainContinuizer()
-            dc.multinomial_treatment = dc.NValues
+            dc = DomainContinuizer()
+            dc.multinomial_treatment = self.multinomial_treatment
             dc.class_treatment = dc.Ignore
             dc.continuous_treatment = \
                     dc.NormalizeBySpan if self.normalization else dc.Leave
@@ -878,7 +901,9 @@ class MultiClassSVMLearner(Orange.core.LinearLearner):
     __new__ = _orange__new__(base=Orange.core.LinearLearner)
 
     def __init__(self, C=1.0, eps=0.01, bias=1.0,
-                 normalization=True, **kwargs):
+                 normalization=True,
+                 multinomial_treatment=DomainContinuizer.NValues,
+                 **kwargs):
         """\
         :param C: Regularization parameter (default 1.0)
         :type C: float
@@ -895,11 +920,22 @@ class MultiClassSVMLearner(Orange.core.LinearLearner):
             (default True)
         :type normalization: bool
 
+        :param multinomial_treatment: Defines how to handle multinomial
+            features for learning. It can be one of the
+            :class:`~.DomainContinuizer` `multinomial_treatment`
+            constants (default: `DomainContinuizer.NValues`).
+
+        :type multinomial_treatment: int
+
+        .. versionadded:: 2.6.1
+            Added `multinomial_treatment`
+
         """
         self.C = C
         self.eps = eps
         self.bias = bias
         self.normalization = normalization
+        self.multinomial_treatment = multinomial_treatment
         for name, val in kwargs.items():
             setattr(self, name, val)
 
@@ -910,8 +946,8 @@ class MultiClassSVMLearner(Orange.core.LinearLearner):
             raise TypeError("Can only learn a discrete class.")
 
         if data.domain.has_discrete_attributes(False) or self.normalization:
-            dc = Orange.data.continuization.DomainContinuizer()
-            dc.multinomial_treatment = dc.NValues
+            dc = DomainContinuizer()
+            dc.multinomial_treatment = self.multinomial_treatment
             dc.class_treatment = dc.Ignore
             dc.continuous_treatment = \
                     dc.NormalizeBySpan if self.normalization else dc.Leave
@@ -1156,11 +1192,11 @@ MeasureAttribute_SVMWeights = ScoreSVMWeights
 
 
 class RFE(object):
-
-    """Iterative feature elimination based on weights computed by
+    """
+    Iterative feature elimination based on weights computed by a
     linear SVM.
 
-    Example::
+    Example:
 
         >>> table = Orange.data.Table("promoters.tab")
         >>> svm_l = Orange.classification.svm.SVMLearner(
@@ -1175,8 +1211,13 @@ class RFE(object):
 
     def __init__(self, learner=None):
         """
-        :param learner: A linear svm learner for use with
-            :class:`ScoreSVMWeights`.
+        :param learner: A linear svm learner for use for scoring (this
+            learner is passed to :class:`ScoreSVMWeights`)
+
+        :type learner: :class:`LinearSVMLearner` or :class:`SVMLearner` with
+            linear kernel
+
+        .. seealso:: :class:`ScoreSVMWeights`
 
         """
         self.learner = learner
