@@ -40,6 +40,8 @@ Submodules
 
 .. automodule:: Orange.utils.selection
 
+.. automodule:: Orange.utils.serverfiles
+
 """
 
 __all__ = ["deprecated_members", "deprecated_keywords",
@@ -696,3 +698,5 @@ def wget(url, directory=".", dst_obj=None, progress=None):
 import warnings
 
 import selection
+
+import render
