@@ -28,10 +28,11 @@
 #include "spec_gen.hpp"
 #include "table.hpp"
 
-#include "assoc.ppp"
+#include "assoc.hpp"
 
 DEFINE_TOrangeVector_classDescription(PAssociationRule, "TAssociationRules", true, ORANGE_API)
 
+#include "assoc.ppp"
 
 TItemSetValue::TItemSetValue(int al)
 : value(al),
@@ -179,8 +180,11 @@ int TAssociationRule::countItems(PExample ex)
 
 bool TAssociationRule::applies(const TExample &ex, const PExample &side)
 {
-  if (side->domain->variables->size())
-    return side->compatible(ex);
+  if (side->domain->variables->size()){
+    //For unpickled rules to work.
+    TExample converted_ex(side->domain, ex);
+    return side->compatible(converted_ex);
+  }
 
   // all meta-attributes that appear in 'side' must also appear in 'ex' and be noSpecial
   const_ITERATE(TMetaValues, mi, side->meta) {
@@ -298,7 +302,7 @@ PAssociationRules TAssociationRulesInducer::operator()(PExampleGenerator example
 
   PITERATE(TVarList, vi, examples->domain->variables)
     if ((*vi)->varType != TValue::INTVAR)
-      raiseError("cannot induce rules with non-discrete attributes (such as '%s')", (*vi)->name.c_str());
+      raiseError("cannot induce rules with non-discrete attributes (such as '%s')", (*vi)->get_name().c_str());
 
   TItemSetNode *tree = NULL;
   PAssociationRules rules;
@@ -385,7 +389,7 @@ int TAssociationRulesInducer::buildTree1(PExampleGenerator gen, const int &weigh
       if (!(*exi).isSpecial()) {
         const int exv = (*exi).intV;
         if (exv >= (int)(ip->values.size()))
-          raiseError("invalid value of attribute '%s'", gen->domain->variables->at(exi-(*ei).begin())->name.c_str());
+          raiseError("invalid value of attribute '%s'", gen->domain->variables->at(exi-(*ei).begin())->get_name().c_str());
         ip->values[(*exi).intV].examples.push_back(TExWei(index, wex));
       }
     }
