@@ -93,7 +93,7 @@ void TBasicAttrStat::add(float f, float p)
   n += p;
   if (!holdRecomputation && (n>0)) {
     avg = sum/n;
-    dev = sqrt(sum2/n - avg*avg);
+    dev = sqrt(std::max(sum2/n - avg*avg, 0.0f));
   }
 
   if (f<min) 
@@ -105,7 +105,7 @@ void TBasicAttrStat::add(float f, float p)
 void TBasicAttrStat::recompute()
 { if (n>0) {
     avg = sum/n;
-    dev = sqrt(sum2/n - avg*avg);
+    dev = sqrt(std::max(sum2/n - avg*avg, 0.0f));
   }
   else
     avg = dev = -1;
@@ -202,6 +202,12 @@ TPearsonCorrelation::TPearsonCorrelation(PExampleGenerator gen, PVariable v1, PV
   
   t = r*sqrt((N-2)/(1-r*r));
   df = int(floor(N));
-  
-  p=betai(double(df*0.5), double(0.5), double(df/(df+t*t)));
+  try
+  {
+    p=betai(double(df*0.5), double(0.5), double(df/(df+t*t)));
+  }
+    catch (const std::exception &exc)
+    {
+      raiseError(exc.what());
+    }
 }
