@@ -27,10 +27,12 @@
 #include "learn.hpp"
 #include "getarg.hpp"
 
-#include "c4.5.ppp"
+#include "c4.5.hpp"
 
 DEFINE_TOrangeVector_classDescription(PC45TreeNode, "TC45TreeNodeList", true, ORANGE_API)
 
+#include "c4.5.ppp"
+
 bool c45Loaded = false;
 
 typedef void *learnFunc(int trials, char gainRatio, char subset, char batch, char probThresh,
@@ -280,8 +282,8 @@ bool TC45Learner::convertDomain(PDomain dom)
   PITERATE(TVarList, vi, dom->attributes) {
     *(SpecialStatusi++) = NULL;
 
-    *AttNamei = mlnew char[(*vi)->name.length()+1];
-    strcpy(*(AttNamei++), (*vi)->name.c_str());
+    *AttNamei = mlnew char[(*vi)->get_name().length()+1];
+    strcpy(*(AttNamei++), (*vi)->get_name().c_str());
 
     if ((*vi)->varType==TValue::INTVAR) {
       int noOfValues = (*vi).AS(TEnumVariable)->noOfValues();
@@ -704,16 +706,19 @@ PTreeNode TC45TreeNode::asTreeNode(PExampleGenerator examples, const int &weight
     branchSizes->addint(i++, (*li)->items);
   newNode->branchSizes = branchSizes;
     
-  TEnumVariable *dummyVar = mlnew TEnumVariable(tested->name);
+  TEnumVariable *dummyVar = mlnew TEnumVariable(tested->get_name());
   PVariable wdummyVar = dummyVar;
 
   switch (nodeType) {
-    case Branch:
-      newNode->branchSelector = mlnew TClassifierFromVar(tested, branchSizes);
+	case Branch: {
+	  TClassifierFromVar *cfv = mlnew TClassifierFromVar(tested, branchSizes);
+	  cfv->transformUnknowns = false;
+      newNode->branchSelector = cfv;
       newNode->branchDescriptions = mlnew TStringList(tested.AS(TEnumVariable)->values.getReference());
       break;
+	}
 
-    case Cut:
+	case Cut: {
       newNode->branchDescriptions = mlnew TStringList;
 
       char str[128];
@@ -724,10 +729,14 @@ PTreeNode TC45TreeNode::asTreeNode(PExampleGenerator examples, const int &weight
       newNode->branchDescriptions->push_back(str);
       dummyVar->values->push_back(str);
 
-      newNode->branchSelector = mlnew TClassifierFromVar(wdummyVar, tested, branchSizes, mlnew TThresholdDiscretizer(cut));
+	  
+	  TClassifierFromVar *cfv = mlnew TClassifierFromVar(wdummyVar, tested, branchSizes, mlnew TThresholdDiscretizer(cut));
+	  cfv->transformUnknowns = false;
+      newNode->branchSelector = cfv;
       break;
+    }
 
-    case Subset:
+	case Subset: {
       int noval = 1 + *max_element(mapping->begin(), mapping->end());
       dummyVar->values = mlnew TStringList(noval, "");
       TStringList::const_iterator tvi(tested.AS(TEnumVariable)->values->begin());
@@ -744,9 +753,12 @@ PTreeNode TC45TreeNode::asTreeNode(PExampleGenerator examples, const int &weight
         if ((*vi).find(",") != string::npos)
           *vi = "in [" + *vi + "]";
 
-      newNode->branchSelector = mlnew TClassifierFromVar(wdummyVar, tested, branchSizes,mlnew TMapIntValue(mapping));
+	  TClassifierFromVar *cfv = mlnew TClassifierFromVar(wdummyVar, tested, branchSizes,mlnew TMapIntValue(mapping));
+	  cfv->transformUnknowns = false;
+      newNode->branchSelector = cfv;
       newNode->branchDescriptions = dummyVar->values;
       break;
+	}
   }
 
   vector<int> newWeights;
