@@ -38,7 +38,7 @@ extern TMLClassDefinition MLDef_Variable;
 #include "externs.px"
 
 
-DATASTRUCTURE(Example, TPyExample, 0)
+DATASTRUCTURE(Example - Orange.data.Instance, TPyExample, 0)
 
 
 bool convertFromPythonExisting(PyObject *lst, TExample &example)
@@ -59,21 +59,26 @@ bool convertFromPythonExisting(PyObject *lst, TExample &example)
     return false;
   }
 
-  if (Py_ssize_t(dom->variables->size()) != PyList_Size(lst)) {
-    PyErr_Format(PyExc_IndexError, "invalid list size (%i items expected)", dom->variables->size());
+  int const nvars = dom->variables->size() + dom->classVars->size();
+  if (Py_ssize_t(nvars) != PyList_Size(lst)) {
+    PyErr_Format(PyExc_IndexError, "invalid list size (got %i, expected %i items)",
+        PyList_Size(lst), nvars);
     return false;
   }
 
   Py_ssize_t pos = 0;
   TExample::iterator ei(example.begin());
-  PITERATE(TVarList, vi, dom->variables) {
+  TVarList::iterator vi(dom->variables->begin());
+  TVarList::const_iterator const ve(dom->variables->end());
+  TVarList::const_iterator const ce(dom->classVars->end());
+  while(vi != ce && vi != ve) {
     PyObject *li=PyList_GetItem(lst, pos++);
     if (!li)
       PYERROR(PyExc_SystemError, "can't read the list", false);
 
     if (PyOrValue_Check(li))
       if (PyValue_AS_Variable(li) ? (PyValue_AS_Variable(li) != *vi) : (PyValue_AS_Value(li).varType=!(*vi)->varType) ) {
-        PyErr_Format(PyExc_TypeError, "wrong value type for attribute no. %i (%s)", pos, (*vi)->name.c_str());
+        PyErr_Format(PyExc_TypeError, "wrong value type for attribute no. %i (%s)", pos, (*vi)->get_name().c_str());
         return false;
       }
       else
@@ -91,7 +96,7 @@ bool convertFromPythonExisting(PyObject *lst, TExample &example)
         if (PyInt_Check(li))
           *(ei++)=TValue(int(PyInt_AsLong(li)));
         else {
-          PyErr_Format(PyExc_TypeError, "attribute no. %i (%s) is ordinal, string value expected", pos, (*vi)->name.c_str());
+          PyErr_Format(PyExc_TypeError, "attribute no. %i (%s) is ordinal, string value expected", pos, (*vi)->get_name().c_str());
           return false;
         }
       }
@@ -100,13 +105,16 @@ bool convertFromPythonExisting(PyObject *lst, TExample &example)
         if (PyNumber_ToFloat(li, f))
           *(ei++) = TValue(f);
         else {
-          PyErr_Format(PyExc_TypeError, "attribute no. %i (%s) is continuous, float value expected", pos, (*vi)->name.c_str());
+          PyErr_Format(PyExc_TypeError, "attribute no. %i (%s) is continuous, float value expected", pos, (*vi)->get_name().c_str());
           return false;
         }
       }
       else
         ei++;
     }
+    if (++vi == ve) {
+        vi = dom->classVars->begin();
+    }
   }
 
   return true;
@@ -193,7 +201,7 @@ int Example_clear(TPyExample *self)
 
 bool readBoolFlag(PyObject *keywords, char *flag);
 
-CONSTRUCTOR_KEYWORDS(Example, "filterMetas")
+CONSTRUCTOR_KEYWORDS(Example, "filterMetas filter_metas")
 
 
 PyObject *Example_new(PyTypeObject *type, PyObject *args, PyObject *keywords) BASED_ON(ROOT, "(domain, [list of values])")
@@ -203,7 +211,7 @@ PyObject *Example_new(PyTypeObject *type, PyObject *args, PyObject *keywords) BA
 
     if (PyArg_ParseTuple(args, "O&|O", cc_Domain, &dom, &list)) {
       if (list && PyOrExample_Check(list)) {
-        PExample ex = mlnew TExample(dom, PyExample_AS_Example(list).getReference(), readBoolFlag(keywords, "filterMetas"));
+        PExample ex = mlnew TExample(dom, PyExample_AS_Example(list).getReference(), readBoolFlag(keywords, "filterMetas") || readBoolFlag(keywords, "filter_metas"));
         return Example_FromWrappedExample(ex);
       }
 
@@ -301,11 +309,11 @@ int weightIndex(const TExample &example, PyObject *pyindex)
 
 PyObject *Example_reference(TPyExample *pex) PYARGS(METH_NOARGS, "unique reference (pointer to) the object")
 {
-  return PyInt_FromLong(int(&PyExample_AS_ExampleReference(pex)));
+  return PyInt_FromSize_t(size_t(&PyExample_AS_ExampleReference(pex)));
 }
 
 
-PyObject *Example_getweight(TPyExample *pex, PyObject *pyindex) PYARGS(METH_O, "(id) -> weight; Returns example's weight")
+PyObject *Example_get_weight(TPyExample *pex, PyObject *pyindex) PYARGS(METH_O, "(id) -> weight; Returns example's weight")
 {
   PyTRY
     const TExample &example = PyExample_AS_ExampleReference(pex);
@@ -325,7 +333,7 @@ PyObject *Example_getweight(TPyExample *pex, PyObject *pyindex) PYARGS(METH_O, "
 }
 
 
-PyObject *Example_setweight(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS, "(id[, weight]); Sets example's weight to given value")
+PyObject *Example_set_weight(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS, "(id[, weight]); Sets example's weight to given value")
 { PyTRY
     PyObject *pyindex;
     float weight = 1;
@@ -350,7 +358,7 @@ PyObject *Example_setweight(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS
 }
 
 
-PyObject *Example_removeweight(TPyExample *pex, PyObject *pyindex) PYARGS(METH_O, "(id); Removes examples's weight")
+PyObject *Example_remove_weight(TPyExample *pex, PyObject *pyindex) PYARGS(METH_O, "(id); Removes examples's weight")
 { PyTRY
     TExample &example = PyExample_AS_ExampleReference(pex);
     int index = weightIndex(example, pyindex);
@@ -369,7 +377,7 @@ PyObject *Example_removeweight(TPyExample *pex, PyObject *pyindex) PYARGS(METH_O
 }
 
 
-PyObject *Example_getmeta(TPyExample *pex, PyObject *index) PYARGS(METH_O, "(id | var) -> Value; Gets a meta-value")
+PyObject *Example_get_meta(TPyExample *pex, PyObject *index) PYARGS(METH_O, "(id | var) -> Value; Gets a meta-value")
 { PyTRY
     PVariable var;
     int idx = getMetaIdFromPy(PyExample_AS_Example(pex), index, var);
@@ -384,7 +392,7 @@ PyObject *Example_getmeta(TPyExample *pex, PyObject *index) PYARGS(METH_O, "(id
 }
 
 
-PyObject *Example_getmetas(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS, "([key-type]) -> dictionary with a copy of example's meta attributes")
+PyObject *Example_get_metas(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS, "([key-type]) -> dictionary with a copy of example's meta attributes")
 {
   PyTRY
     PyObject *pyoptional = NULL;
@@ -435,7 +443,7 @@ PyObject *Example_getmetas(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS,
           if (!variable)
             continue;
           if (keytype == &PyString_Type)
-            key = PyString_FromString(variable->name.c_str());
+            key = PyString_FromString(variable->get_name().c_str());
           else
             key = WrapOrange(variable);
         }
@@ -458,7 +466,7 @@ PyObject *Example_getmetas(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS,
 }
 
 
-PyObject *Example_hasmeta(TPyExample *pex, PyObject *index) PYARGS(METH_O, "(id | var) -> bool")
+PyObject *Example_has_meta(TPyExample *pex, PyObject *index) PYARGS(METH_O, "(id | var) -> bool")
 { PyTRY
     PVariable var;
     int idx = getMetaIdFromPy(PyExample_AS_Example(pex), index, var);
@@ -468,7 +476,7 @@ PyObject *Example_hasmeta(TPyExample *pex, PyObject *index) PYARGS(METH_O, "(id
 }
 
 
-PyObject *Example_setvalue(TPyExample *pex, PyObject *vala) PYARGS(METH_O, "(Value) -> None")
+PyObject *Example_set_value(TPyExample *pex, PyObject *vala) PYARGS(METH_O, "(Value) -> None")
 { PyTRY
     if (!PyOrValue_Check(vala))
       PYERROR(PyExc_TypeError, "Example.setvalue: orange.Value expected", PYNULL);
@@ -490,7 +498,7 @@ PyObject *Example_setvalue(TPyExample *pex, PyObject *vala) PYARGS(METH_O, "(Val
 }
 
 
-PyObject *Example_setmeta(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS, "(Value, int) | (variable, value); Sets a meta-value")
+PyObject *Example_set_meta(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS, "(Value, int) | (variable, value); Sets a meta-value")
 { PyTRY
     PExample example=PyExample_AS_Example(pex);
   
@@ -541,7 +549,7 @@ PyObject *Example_setmeta(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS,
 
       else {
         TMetaDescriptor *desc=example->domain->metas[
-            PyOrVariable_Check(par1) ? PyOrange_AsVariable(par1)->name
+            PyOrVariable_Check(par1) ? PyOrange_AsVariable(par1)->get_name()
                                      : string(PyString_AsString(par1))];
         if (!desc)
           PYERROR(PyExc_TypeError, "invalid variable", PYNULL);
@@ -566,7 +574,7 @@ PyObject *Example_setmeta(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS,
 }
   
 
-PyObject *Example_removemeta(TPyExample *pex, PyObject *index) PYARGS(METH_O, "(id); Removes a meta-value")
+PyObject *Example_remove_meta(TPyExample *pex, PyObject *index) PYARGS(METH_O, "(id); Removes a meta-value")
 { PyTRY
     PVariable var;
     int idx = getMetaIdFromPy(PyExample_AS_Example(pex), index, var);
@@ -580,7 +588,7 @@ PyObject *Example_removemeta(TPyExample *pex, PyObject *index) PYARGS(METH_O, "(
 
 
 
-PyObject *Example_getclass(TPyExample *pex) PYARGS(METH_NOARGS, "()  -> Value; Returns example's class")
+PyObject *Example_get_class(TPyExample *pex) PYARGS(METH_NOARGS, "()  -> Value; Returns example's class")
 { PyTRY
       const TExample &example = PyExample_AS_ExampleReference(pex);
       const PVariable &classVar = example.domain->classVar;
@@ -593,7 +601,22 @@ PyObject *Example_getclass(TPyExample *pex) PYARGS(METH_NOARGS, "()  -> Value; R
 }
 
 
-PyObject *Example_setclass(TPyExample *pex, PyObject *val) PYARGS(METH_O, "(value); Sets example's class")
+PyObject *Example_get_classes(TPyExample *pex) PYARGS(METH_NOARGS, "()  -> [Values]; Returns example's class")
+{ PyTRY
+      const TExample &example = PyExample_AS_ExampleReference(pex);
+      PyObject *list=PyList_New(0);
+      TExample::const_iterator ei=example.values_end;
+      const_PITERATE(TVarList, vi, example.domain->classVars) {
+          PyObject *valo = Value_FromVariableValue(*vi, *ei++);
+          PyList_Append(list, valo);
+          Py_DECREF(valo);
+      }
+      return list;
+  PyCATCH
+}
+
+
+PyObject *Example_set_class(TPyExample *pex, PyObject *val) PYARGS(METH_O, "(value); Sets example's class")
 { PyTRY
     PExample &example=PyExample_AS_Example(pex);
     PVariable &classVar = example->domain->classVar;
@@ -611,6 +634,28 @@ PyObject *Example_setclass(TPyExample *pex, PyObject *val) PYARGS(METH_O, "(valu
 }
 
 
+PyObject *Example_set_classes(TPyExample *pex, PyObject *val) PYARGS(METH_O, "(list-of-Values); Returns example's class")
+{ PyTRY
+      const TExample &example = PyExample_AS_ExampleReference(pex);
+      if (!PyList_Check(val)) {
+          PYERROR(PyExc_TypeError, "list of values expected", PYNULL);
+      }
+      if (PyList_Size(val) != example.domain->classVars->size()) {
+          PyErr_Format(PyExc_IndexError, "expected %i values, got %i", example.domain->classVars->size(), PyList_Size(val));
+          return NULL;
+      }
+      TExample::iterator ei=example.values_end;
+      int pos = 0;
+      const_PITERATE(TVarList, vi, example.domain->classVars) {
+          if (!convertFromPython(PyList_GET_ITEM(val, pos++), *ei++, *vi))
+              return PYNULL;
+      }
+      RETURN_NONE;
+  PyCATCH
+}
+
+
+
 PyObject *Example_compatible(TPyExample *pex, PyObject *args) PYARGS(METH_VARARGS, "(example[, ignoreClass]); Returns true if examples are compatible")
 { PyTRY
     PExample example;
@@ -800,7 +845,7 @@ PyObject *convertToPythonNative(const TExample &example, int natvt, bool tuples,
 }
 
 
-PyObject *Example_native(TPyExample *pex, PyObject *args, PyObject *keyws) PYARGS(METH_VARARGS | METH_KEYWORDS, "([nativity, tuple=, substituteDC=, substituteDK=, substituteOther=])  -> list; Converts an example to a list")
+PyObject *Example_native(TPyExample *pex, PyObject *args, PyObject *keyws) PYARGS(METH_VARARGS | METH_KEYWORDS, "([nativity, tuple=, substitute_DC=, substitute_DK=, substitute_Other=])  -> list; Converts an example to a list")
 { PyTRY
     int natvt=1;
     if (args && !PyArg_ParseTuple(args, "|i", &natvt))
@@ -810,10 +855,21 @@ PyObject *Example_native(TPyExample *pex, PyObject *args, PyObject *keyws) PYARG
     PyObject *pytuples = keyws ? PyDict_GetItemString(keyws, "tuple") : PYNULL;
     tuples = pytuples && (PyObject_IsTrue(pytuples) != 0);
 
-    PyObject *forDC = keyws ? PyDict_GetItemString(keyws, "substituteDC") : PYNULL;
-    PyObject *forDK = keyws ? PyDict_GetItemString(keyws, "substituteDK") : PYNULL;
-    PyObject *forSpecial = keyws ? PyDict_GetItemString(keyws, "substituteOther") : PYNULL;
-
+    PyObject *forDC = NULL, *forDK = NULL, *forSpecial = NULL;
+    if (keyws) {
+        forDC = PyDict_GetItemString(keyws, "substitute_DC");
+        if (!forDC) {
+            forDC = PyDict_GetItemString(keyws, "substituteDC");
+        }
+        forDC = PyDict_GetItemString(keyws, "substitute_DK");
+        if (!forDC) {
+            forDC = PyDict_GetItemString(keyws, "substituteDK");
+        }
+        forDC = PyDict_GetItemString(keyws, "substitute_other");
+        if (!forDC) {
+            forDC = PyDict_GetItemString(keyws, "substituteOther");
+        }
+    }
     return convertToPythonNative(PyExample_AS_ExampleReference(pex), natvt, tuples, forDK, forDC, forSpecial);
   PyCATCH
 }
@@ -827,7 +883,7 @@ PyObject *Example_checksum(TPyExample *pex, PyObject *) PYARGS(METH_NOARGS, "()
 
 void Example_pack(const TExample &example, TCharBuffer &buf, PyObject *&otherValues)
 {
-  for(TValue *vali = example.values; vali != example.values_end; vali++)
+  for(TValue *vali = example.values; vali != example.classes_end; vali++)
     Value_pack(*vali, buf, otherValues);
 
   buf.writeInt(example.meta.size() | (example.name ? 1<<31 : 0));
@@ -851,8 +907,13 @@ void Example_pack(const TExample &example, TCharBuffer &buf, PyObject *&otherVal
 
 void Example_unpack(TExample &example, TCharBuffer &buf, PyObject *&otherValues, int &otherValuesIndex)
 {
-  TVarList::const_iterator vi = example.domain->variables->begin();
-  for(TValue *vali = example.values; vali != example.values_end; vali++, vi++) {
+  TVarList::const_iterator vi;
+  TValue *vali = example.values;
+  for(vi = example.domain->variables->begin(); vali != example.values_end; vali++, vi++) {
+    vali->varType = (*vi)->varType;
+    Value_unpack(*vali, buf, otherValues, otherValuesIndex);
+  }
+  for(vi = example.domain->classVars->begin(); vali != example.classes_end; vali++, vi++) {
     vali->varType = (*vi)->varType;
     Value_unpack(*vali, buf, otherValues, otherValuesIndex);
   }
@@ -937,20 +998,34 @@ string TPyExample2string(TPyExample *pex)
 { PExample example = PyExample_AS_Example(pex);
   string res("[");
   TVarList::iterator vi(example->domain->variables->begin());
-  PITERATE(TExample, ei, example) {
-    if (ei!=example->begin())
+  TVarList::iterator ve(example->domain->variables->end());
+  TExample::const_iterator ei = example->values;
+  for(; vi != ve; vi++, ei++) {
+    if (ei != example->values)
       res+=", ";
-    addValue(res, *ei, *(vi++));
+    addValue(res, *ei, *vi);
   }
   res+="]";
 
+  vi = example->domain->classVars->begin();
+  ve = example->domain->classVars->end();
+  if (vi != ve) {
+      res += " (";
+      for(; vi != ve; vi++, ei++) {
+          if (ei != example->values_end)
+              res += ", ";
+          addValue(res, *ei, *vi);
+      }
+      res += ")";
+  }
+
   int madded=0;
   ITERATE(TMetaValues, mi, example->meta) {
     res+= (madded++) ? ", " : ", {";
     
     TMetaDescriptor *desc=example->domain->metas[(*mi).first];
     if (desc) {
-      res+="\""+desc->variable->name+"\":";
+      res+="\""+desc->variable->get_name()+"\":";
       addValue(res, (*mi).second, desc->variable);
     }
     else
@@ -1000,10 +1075,24 @@ PyObject *Example_newId(TPyExample *self)
   PyCATCH
 }
 
-int Example_cmp(TPyExample *one, TPyExample *another)
+PyObject *richcmp_from_sign(const int &i, const int &op);
+
+PyObject *Example_richcmp(TPyExample *one, TPyExample *another, int op)
 { PyTRY
-    return PyExample_AS_Example(one)->compare(PyExample_AS_ExampleReference(another));
-  PyCATCH_1
+    PExample rone = PyExample_AS_Example(one);
+    TExample &ranother = PyExample_AS_ExampleReference(another);
+    if (rone->domain != ranother.domain) {
+        if ((op == Py_EQ) || (op == Py_NE)) {
+            PyObject *res = op == Py_NE ? Py_True : Py_False;
+            Py_INCREF(res);
+            return res;
+        }
+        else {
+            PYERROR(PyExc_ValueError, "examples are from different domains", NULL);
+        }
+    }
+    return richcmp_from_sign(rone->compare(ranother), op);
+  PyCATCH
 }
 
 
@@ -1021,9 +1110,34 @@ int Example_len(TPyExample *pex)
 }
 
 
+char *example_underscores[][2] = {
+    {"getweight", "get_weight"},
+    {"setweight", "set_weight"},
+    {"removeweight", "remove_weight"},
+    {"getmeta", "get_meta"},
+    {"getmetas", "get_metas"},
+    {"hasmetas", "has_meta"},
+    {"setvalue", "set_value"},
+    {"setmeta", "set_meta"},
+    {"removemeta", "remove_meta"},
+    {"getclass", "get_class"},
+    {"setclass", "set_class"},
+    {NULL, NULL}
+};
+
 PyObject *Example_getattr(TPyExample *self, PyObject *name)
 {
-  if (!PyString_Check(name) || strcmp(PyString_AsString(name), "name"))
+  char *orig = PyString_AsString(name);
+  for(char *(*ei)[2] = example_underscores; **ei; ei++) {
+      if (!strcmp(orig, **ei)) {
+          PyObject *trans = PyString_FromString((*ei)[1]);
+          PyObject *value = PyObject_GenericGetAttr((PyObject *)self, trans);
+          Py_DECREF(trans);
+          return value;
+      }
+  }
+
+  if (!PyString_Check(name) || strcmp(orig, "name"))
     return PyObject_GenericGetAttr((PyObject *)self, name);
 
   const string *ename = self->example->name;
