@@ -34,7 +34,7 @@
 
 
 BASED_ON(Orange, ROOT)
-DATASTRUCTURE(Orange, TPyOrange, orange_dict)
+DATASTRUCTURE(Orange - Orange.core.OrangeBase, TPyOrange, orange_dict)
 RECOGNIZED_ATTRIBUTES(Orange, "name shortDescription description")
 
 
@@ -278,14 +278,56 @@ PyObject *PyOrange__members__(TPyOrange *self)
 }
 
 
+char *camel2underscore(const char *camel)
+{
+    const char *ci = camel;
+    if ((*ci >= 'A') && (*ci <= 'Z')) {
+        return NULL;
+    }
+
+    char *underscored = (char *)malloc(2*strlen(camel)+1);
+    char *ui = underscored;
+    bool changed = false;
+    *ui = *ci;
+    while(*ci) { // just copied
+        if (   (*ci >= 'a') && (*ci <= 'z')       // a small letter
+            && (ci[1] >= 'A') && (ci[1] <= 'Z')) {  // followed by capital
+            *++ui = '_';
+            const char of = (ci[2] < 'A') || (ci[2] > 'Z') ? 32 : 0; // 32, if not followed by capital 
+            *++ui = *++ci + of;
+            changed = true;
+        }
+        else {
+            *++ui = *++ci;
+        }
+    }
+    if (!changed) {
+        free(underscored);
+        underscored = NULL;
+    }
+    return underscored;
+}
+
+
 PyObject *PyOrange_translateObsolete(PyObject *self, PyObject *pyname)
 { 
-  char *name=PyString_AsString(pyname);
-  for(TOrangeType *selftype = PyOrange_OrangeBaseClass(self->ob_type); PyOrange_CheckType((PyTypeObject *)selftype); selftype=(TOrangeType *)(selftype->ot_inherited.tp_base))
-    if (selftype->ot_aliases)
-      for(TAttributeAlias *aliases=selftype->ot_aliases; aliases->alias; aliases++)
-        if (!strcmp(name, aliases->alias))
-          return PyString_FromString(aliases->realName);
+  char *name = PyString_AsString(pyname);
+  char *underscored = camel2underscore(name);
+  for(TOrangeType *selftype = PyOrange_OrangeBaseClass(self->ob_type); PyOrange_CheckType((PyTypeObject *)selftype); selftype=(TOrangeType *)(selftype->ot_inherited.tp_base)) {
+      if (selftype->ot_aliases) {
+          for(TAttributeAlias *aliases=selftype->ot_aliases; aliases->alias; aliases++) {
+              if (!strcmp(name, aliases->alias) || (underscored && !strcmp(underscored, aliases->alias))) {
+                  if (underscored) {
+                      free(underscored);
+                  }
+                  return PyString_FromString(aliases->realName);
+              }
+          }
+      }
+  }
+  if (underscored) {
+      free(underscored);
+  }
   return NULL;
 }    
 
@@ -350,9 +392,6 @@ PyObject *Orange_getattr1(TPyOrange *self, const char *name)
       {}
     }
  
-    if (!strcmp(name, "name") || !strcmp(name, "shortDescription") || !strcmp(name, "description"))
-      return PyString_FromString("");
-
     PyErr_Format(PyExc_AttributeError, "'%s' has no attribute '%s'", self->ob_type->tp_name, name);
     return PYNULL;
   PyCATCH;
@@ -730,15 +769,41 @@ PyObject *Orange_getattr(TPyOrange *self, PyObject *name)
 { 
   PyTRY
     PyObject *res = Orange_getattr1(self, name);
+    char *underscored = NULL;
     if (!res) {
-      PyObject *translation = PyOrange_translateObsolete((PyObject *)self, name);
-      if (translation) {
-        PyErr_Clear();
-        res = Orange_getattr1(self, translation);
-        Py_DECREF(translation);
-      }
+        char *camel = PyString_AsString(name);
+        underscored = camel2underscore(camel);
+        if (underscored) {
+            PyObject *translation = PyString_FromString(underscored);
+            PyErr_Clear();
+            res = Orange_getattr1(self, translation);
+            Py_DECREF(translation);
+        }
+    }
+    if (!res) {
+        PyObject *translation = PyOrange_translateObsolete((PyObject *)self, name);
+        if (translation) {
+            PyErr_Clear();
+            res = Orange_getattr1(self, translation);
+            Py_DECREF(translation);
+        }
     }
 
+    if (!res && underscored) {
+        PyMethodDef *mi = self->ob_type->tp_methods;
+        if (mi) {
+            for(; mi->ml_name; mi++) {
+                if (!strcmp(underscored, mi->ml_name)) {
+                    res = PyMethod_New((PyObject *)mi->ml_meth, (PyObject *)self, (PyObject *)(self->ob_type));
+                    break;
+                }
+            }
+        }
+    }
+
+    if (underscored) {
+        free(underscored);
+    }
     return res;
   PyCATCH
 }
@@ -801,6 +866,18 @@ int Orange_setattrLow(TPyOrange *self, PyObject *pyname, PyObject *args, bool wa
     if (res!=1)
       return res;
     
+    PyErr_Clear();
+    char *camel = PyString_AsString(pyname);
+    char *underscored = camel2underscore(camel);
+    if (underscored) {
+        PyObject *translation = PyString_FromString(underscored);
+        free(underscored);
+        res = Orange_setattr1(self, translation, args);
+        Py_DECREF(translation);
+    }
+    if (res!=1)
+      return res;
+
     PyErr_Clear();
     // Try to translate it as an obsolete alias for C++ class member
     PyObject *translation = PyOrange_translateObsolete((PyObject *)self, pyname);
@@ -809,7 +886,7 @@ int Orange_setattrLow(TPyOrange *self, PyObject *pyname, PyObject *args, bool wa
       char *name = PyString_AsString(pyname);
       char *transname = PyString_AsString(translation);
       sprintf(sbuf, "'%s' is an (obsolete) alias for '%s'", name, transname);
-      if (PyErr_Warn(PyExc_OrangeAttributeWarning, sbuf))
+      if (PyErr_Warn(PyExc_DeprecationWarning, sbuf))
         return -1;
         
       res = Orange_setattr1(self, translation, args);
@@ -944,6 +1021,74 @@ PyObject *Orange_str(TPyOrange *self)
 }
 
 
+char const *genericNames[] = {"Classifier", "Learner", "Discretizer", NULL};
+
+PyObject *Orange_get_name(TPyOrange *self)
+{
+  PyTRY
+    PyObject *pyname = Orange_getattr1(self, "name");
+    if (!pyname) { 
+      PyErr_Clear();
+      if (self->orange_dict) {
+        pyname = PyDict_GetItemString(self->orange_dict, "name");
+        Py_XINCREF(pyname);
+      }
+    }
+    if (pyname) {
+      if (PyString_Check(pyname)) {
+        return pyname;
+      }
+      PyObject *pystr = PyObject_Repr(pyname);
+      Py_DECREF(pyname);
+      return pystr;
+    }
+
+    char const *tp_name = self->ob_type->tp_name;
+    char const *dotp = tp_name + strlen(tp_name);
+    while((dotp != tp_name) && (*dotp != '.')) {
+      dotp--;
+    }
+	if (*dotp == '.') {
+	  dotp++;
+	}
+	if (*dotp == '_') {
+	  dotp++;
+	}
+    char *name = (char *)malloc(strlen(dotp) + 1);
+    strcpy(name, dotp);
+
+	const int lenname = strlen(name);
+	char *nameend = name + lenname;
+    for(char const *const *gen = genericNames; *gen; gen++) {
+		if (strlen(*gen) < lenname) {
+		    char *cap = nameend - strlen(*gen);
+		    if (!strcmp(cap, *gen)) {
+              *cap = 0;
+              break;
+			}
+		}
+	}
+    if ((*name >= 'A') && (*name <= 'Z')) {
+      *name ^= 32;
+      }
+    return PyString_FromString(name);
+  PyCATCH
+}
+
+
+
+int Orange_set_name(TPyOrange *self, PyObject *arg)
+{
+  PyTRY
+    int res = Orange_setattr1(self, "name", arg);
+    if (res == 1) {
+        res = Orange_setattrDictionary(self, "name", arg, false);
+    }
+    return res;
+  PyCATCH_1
+}
+
+
 int Orange_nonzero(PyObject *self)
 { PyTRY
     if (self->ob_type->tp_as_sequence && self->ob_type->tp_as_sequence->sq_length)
