@@ -23,7 +23,7 @@
 #include "contingency.hpp"
 #include "examplegen.hpp"
 
-#include "estimateprob.ppp"
+#include "estimateprob.hpp"
 #include "stat.hpp"
 #include "random.hpp"
 
@@ -31,6 +31,8 @@
 DEFINE_TOrangeVector_classDescription(PProbabilityEstimator, "TProbabilityEstimatorList", true, ORANGE_API)
 DEFINE_TOrangeVector_classDescription(PConditionalProbabilityEstimator, "TConditionalProbabilityEstimatorList", true, ORANGE_API)
 
+#include "estimateprob.ppp"
+
 TProbabilityEstimator::TProbabilityEstimator(const bool &disc, const bool &cont)
 : supportsDiscrete(disc),
   supportsContinuous(cont)
@@ -262,7 +264,7 @@ PProbabilityEstimator TProbabilityEstimatorConstructor_loess::operator()(PDistri
 { TContDistribution *cdist = frequencies.AS(TContDistribution);
   if (!cdist)
     if (frequencies && frequencies->variable)
-      raiseError("attribute '%s' is not continuous", frequencies->variable->name.c_str());
+      raiseError("attribute '%s' is not continuous", frequencies->variable->get_name().c_str());
     else
       raiseError("continuous distribution expected");
   if (!cdist->size())
@@ -270,7 +272,10 @@ PProbabilityEstimator TProbabilityEstimatorConstructor_loess::operator()(PDistri
 
   map<float, float> loesscurve;
   loess(cdist->distribution, nPoints, windowProportion, loesscurve, distributionMethod);
-  return mlnew TProbabilityEstimator_FromDistribution(mlnew TContDistribution(loesscurve));
+
+  TContDistribution *prob = mlnew TContDistribution(loesscurve);
+  prob->variable = frequencies->variable;
+  return mlnew TProbabilityEstimator_FromDistribution(prob);
 }
 
 
@@ -406,7 +411,7 @@ PConditionalProbabilityEstimator TConditionalProbabilityEstimatorConstructor_ByR
     frequencies = mlnew TContingencyAttrClass(gen, weightID, attrNo);
   if (frequencies->varType != TValue::INTVAR)
     if (frequencies->outerVariable)
-      raiseError("attribute '%s' is not discrete", frequencies->outerVariable->name.c_str());
+      raiseError("attribute '%s' is not discrete", frequencies->outerVariable->get_name().c_str());
     else
       raiseError("discrete attribute for condition expected");
 
@@ -457,7 +462,7 @@ TConditionalProbabilityEstimatorConstructor_loess::TConditionalProbabilityEstima
 PConditionalProbabilityEstimator TConditionalProbabilityEstimatorConstructor_loess::operator()(PContingency frequencies, PDistribution, PExampleGenerator, const long &, const int &) const
 { if (frequencies->varType != TValue::FLOATVAR)
     if (frequencies->outerVariable)
-      raiseError("attribute '%s' is not continuous", frequencies->outerVariable->name.c_str());
+      raiseError("attribute '%s' is not continuous", frequencies->outerVariable->get_name().c_str());
     else
       raiseError("continuous attribute expected for condition");
 
