@@ -42,6 +42,7 @@ long getExampleId()
 TExample::TExample()
 : values(NULL),
   values_end(NULL),
+  classes_end(NULL),
   name(NULL),
   id(getExampleId())
 {}
@@ -51,17 +52,22 @@ TExample::TExample(PDomain dom, bool initMetas)
 : domain(dom),
   values(NULL),
   values_end(NULL),
+  classes_end(NULL),
   name(NULL),
   id(getExampleId())
 { if (!dom)
     raiseError("example needs domain");
 
   const int attrs = domain->variables->size();
-  TValue *vi = values = mlnew TValue[attrs];
+  const int classes = domain->classVars->size();
+  TValue *vi = values = mlnew TValue[attrs+classes];
   values_end = values + attrs;
+  classes_end = values_end + classes;
   PITERATE(TVarList, di, dom->variables)
     *(vi++) = (*di)->DK();
-
+  PITERATE(TVarList, ci, dom->classVars) {
+    *(vi++) = (*ci)->DK();
+  }
   if (initMetas)
     ITERATE(TMetaVector, mi, dom->metas)
       if (!(*mi).optional)
@@ -76,11 +82,14 @@ TExample::TExample(const TExample &orig, bool copyMetas)
   id(orig.id)
 { if (domain) {
     const int attrs = domain->variables->size();
-    TValue *vi = values = mlnew TValue[attrs];
+    const int classes = orig.classes_end - orig.values_end;
+    TValue *vi = values = mlnew TValue[attrs+classes];
     values_end = values + attrs;
-    for(TValue *origi = orig.values, *thisi = values; thisi != values_end; *(thisi++) = *(origi++));
+    classes_end = values_end + classes;
+    for(TValue *origi = orig.values, *thisi = values; thisi != classes_end; *(thisi++) = *(origi++));
   }
-  else values = values_end = NULL;
+  else 
+      values = values_end = classes_end = NULL;
 }
 
 
@@ -93,8 +102,10 @@ TExample::TExample(PDomain dom, const TExample &orig, bool copyMetas)
     raiseError("example needs a domain");
 
   const int attrs = domain->variables->size();
-  values = mlnew TValue[attrs];
+  const int classes = domain->classVars->size();
+  values = mlnew TValue[attrs + classes];
   values_end = values + attrs;
+  classes_end = values_end + classes;
   domain->convert(*this, orig, !copyMetas);
 }
 
@@ -107,7 +118,7 @@ void TExample::insertVal(TValue &srcval, PVariable var, const long &metaID, vect
     // Check if this is an ordinary attribute
     if (position >= 0) {
       if (!mergeTwoValues(values[position], srcval, defined[position]))
-        raiseError("ambiguous value of attribute '%s'", var->name.c_str());
+        raiseError("ambiguous value of attribute '%s'", var->get_name().c_str());
       else
         defined[position] = true;
     }
@@ -116,7 +127,7 @@ void TExample::insertVal(TValue &srcval, PVariable var, const long &metaID, vect
       // Is it meta?
       if (hasMeta(position)) {
         if (!mergeTwoValues(meta[metaID], srcval, true))
-          raiseError("ambiguous value for meta-attribute '%s'", var->name.c_str());
+          raiseError("ambiguous value for meta-attribute '%s'", var->get_name().c_str());
       }
       else
         setMeta(position, srcval);
@@ -146,8 +157,12 @@ TExample::TExample(PDomain dom, PExampleList elist)
 {
   if (!dom)
     raiseError("example needs a domain");
+  if (dom->classVars->size()) {
+      raiseError("example merging does not support multiple classes");
+  }
 
   const int attrs = domain->variables->size();
+  const int classes = domain->classVars->size();
   vector<bool> defined(attrs, false);
 
   TValue *vi = values = mlnew TValue[attrs];
@@ -192,7 +207,7 @@ TExample::~TExample()
 int TExample::traverse(visitproc visit, void *arg) const
 { TRAVERSE(TOrange::traverse);
   
-  for(TValue *vi = values, *ve = values_end; vi!=ve; vi++)
+  for(TValue *vi = values, *ve = classes_end; vi!=ve; vi++)
     if (vi->svalV)
       PVISIT(vi->svalV);
 
@@ -206,7 +221,7 @@ int TExample::traverse(visitproc visit, void *arg) const
 
 int TExample::dropReferences()
 {
-  for(TValue *vi = values, *ve = values_end; vi!=ve; vi++)
+  for(TValue *vi = values, *ve = classes_end; vi!=ve; vi++)
     if (vi->svalV)
       vi->svalV.~PSomeValue();
 
@@ -224,25 +239,27 @@ int TExample::dropReferences()
 TExample &TExample::operator =(const TExample &orig)
 {
   if (!orig.domain) {
-    values = values_end = NULL;
+    if (values)
+       mldelete[] values;
+    values = values_end = classes_end = NULL;
     domain = PDomain();
   }
 
   else {
-    const int attrs = orig.domain->variables->size();
-
+    const int attrs = orig.values_end - orig.values;
+    const int classes = orig.classes_end - orig.values_end;
     if (domain != orig.domain) {
-      if (domain->variables->size() != attrs) {
+      if ((values_end - values != attrs) || (classes_end - values_end != classes)) {
         if (values)
           mldelete[] values;
-        values = mlnew TValue[attrs];
+        values = mlnew TValue[attrs+classes];
         values_end = values + attrs;
+        classes_end = values_end + classes;
       }
-
       domain = orig.domain;
     }
 
-    for(TValue *origi = orig.values, *thisi = values; thisi != values_end; *(thisi++) = *(origi++));
+    for(TValue *origi = orig.values, *thisi = values; thisi != classes_end; *(thisi++) = *(origi++));
   }
 
   meta = orig.meta;
@@ -290,8 +307,8 @@ const TValue &TExample::missingMeta(const int &i) const
     if (md->optional)
       return md->variable->DK();
     else
-      if (md->variable->name.size())
-        raiseError("the value of meta attribute '%s' is missing", md->variable->name.c_str());
+      if (md->variable->get_name().size())
+        raiseError("the value of meta attribute '%s' is missing", md->variable->get_name().c_str());
 
   // no descriptor or no name
   raiseError("meta value with id %i is missing", i);
@@ -312,7 +329,7 @@ bool TExample::operator == (const TExample &other) const
   if (domain != other.domain)
     raiseError("examples are from different domains");
 
-  int Na = domain->variables->size();
+  int Na = domain->variables->size() + domain->classVars->size();
   if (!Na)
     return true;
   for (const_iterator vi1(begin()), vi2(other.begin()); (*vi1==*vi2) && --Na; vi1++, vi2++);
@@ -324,11 +341,19 @@ int TExample::compare(const TExample &other, const bool ignoreClass) const
 { if (domain != other.domain)
     raiseError("examples are from different domains");
 
-  const_iterator i1(begin()), i2(other.begin());
-  int Na = domain->variables->size() - (ignoreClass && domain->classVar ? 1 : 0);
+  int Na = domain->variables->size();
+  if (ignoreClass) {
+      if (domain->classVar) {
+          Na--;
+      }
+  }
+  else {
+      Na += domain->classVars->size();
+  }
   if (!Na)
     return true;
 
+  const_iterator i1(begin()), i2(other.begin());
   int comp;
   while (0==(comp= (*(i1++)).compare(*(i2++))) && --Na);
   return comp;
@@ -339,7 +364,15 @@ bool TExample::compatible(const TExample &other, const bool ignoreClass) const
 { if (domain != other.domain)
     raiseError("examples are from different domains");
   
-  int Na = domain->variables->size() - (ignoreClass && domain->classVar ? 1 : 0);
+  int Na = domain->variables->size();
+  if (ignoreClass) {
+      if (domain->classVar) {
+          Na--;
+      }
+  }
+  else {
+      Na += domain->classVars->size();
+  }
   if (!Na)
     return true;
   for (const_iterator i1(begin()), i2(other.begin()); (*i1).compatible(*i2) && --Na; i1++, i2++);
@@ -349,34 +382,40 @@ bool TExample::compatible(const TExample &other, const bool ignoreClass) const
 
 #include "stringvars.hpp"
 
-void TExample::addToCRC(unsigned long &crc, const bool includeMetas) const
+inline void addToCRC(unsigned int &crc, const PVarList &vars, TValue *&vli)
 {
-  TValue *vli = values;
-  const_PITERATE(TVarList, vi, domain->variables) {
+  const_PITERATE(TVarList, vi, vars) {
     if ((*vi)->varType == TValue::INTVAR)
-      add_CRC((const unsigned long)(vli->isSpecial() ? ILLEGAL_INT : vli->intV), crc);
+      add_CRC(vli->isSpecial() ? ILLEGAL_INT : vli->intV, crc);
     else if (((*vi)->varType == TValue::FLOATVAR))
       add_CRC(vli->isSpecial() ? ILLEGAL_FLOAT : vli->floatV, crc);
     else if ((*vi)->varType == STRINGVAR) {
       if (vli->isSpecial() || !vli->svalV)
-        add_CRC((const unsigned long)ILLEGAL_INT, crc);
+        add_CRC(ILLEGAL_INT, crc);
       else
         add_CRC(vli->svalV.AS(TStringValue)->value.c_str(), crc);
     }
     vli++;
   }
+}
+
+void TExample::addToCRC(unsigned int &crc, const bool includeMetas) const
+{
+  TValue *vli = values;
+  ::addToCRC(crc, domain->variables, vli);
+  ::addToCRC(crc, domain->classVars, vli);
   
   if (includeMetas) {
     const_ITERATE(TMetaValues, mi, meta) {
       add_CRC((const unsigned long)(mi->first), crc);
       const TValue &val = mi->second;
       if (val.varType == TValue::INTVAR)
-        add_CRC((const unsigned long)(val.isSpecial() ? ILLEGAL_INT : val.intV), crc);
+        add_CRC(val.isSpecial() ? ILLEGAL_INT : val.intV, crc);
       else if (val.varType == TValue::INTVAR)
         add_CRC(val.isSpecial() ? ILLEGAL_FLOAT: val.floatV, crc);
       else if (val.varType == STRINGVAR) {
         if (val.isSpecial()  || !vli->svalV)
-          add_CRC((const unsigned long)ILLEGAL_INT, crc);
+          add_CRC(ILLEGAL_INT, crc);
         else
           add_CRC(val.svalV.AS(TStringValue)->value.c_str(), crc);
       }
@@ -386,7 +425,7 @@ void TExample::addToCRC(unsigned long &crc, const bool includeMetas) const
 
 
 int TExample::sumValues(const bool includeMetas) const
-{ unsigned long crc;
+{ unsigned int crc;
   INIT_CRC(crc);
   addToCRC(crc, includeMetas);
   FINISH_CRC(crc);
