@@ -52,6 +52,8 @@ public:
     void swap();
     void permute(const TIntList &newOrder);
 
+    unsigned int size(){ return last - first; }
+
 protected:
     void recursiveMove(const int &offset);
 };
@@ -77,10 +79,30 @@ public:
     PHierarchicalCluster restructure(TClusterW *, PIntList elementIndices, TIntList::iterator &currentElement, int &currentIndex);
 
 private:
-    TClusterW *THierarchicalClustering::merge_CompleteLinkage(TClusterW **clusters, float *callbackMilestones);
-    TClusterW *THierarchicalClustering::merge_SingleLinkage(TClusterW **clusters, float *callbackMilestones);
-    TClusterW *THierarchicalClustering::merge_AverageLinkage(TClusterW **clusters, float *callbackMilestones);
+    TClusterW *merge_CompleteLinkage(TClusterW **clusters, float *callbackMilestones);
+    TClusterW *merge_SingleLinkage(TClusterW **clusters, float *callbackMilestones);
+    TClusterW *merge_AverageLinkage(TClusterW **clusters, float *callbackMilestones);
     // Average linkage also computes Ward's linkage
+
+private:
+    PHierarchicalCluster order_leaves(PHierarchicalCluster root, PSymMatrix matrix, PProgressCallback progress_callback);
+};
+
+
+/*
+ * Optimal leaf ordering.
+ */
+
+
+
+class ORANGE_API THierarchicalClusterOrdering: public TOrange {
+public:
+	__REGISTER_CLASS
+
+	PProgressCallback progress_callback; //P progress callback function
+
+	PHierarchicalCluster operator() (PHierarchicalCluster root, PSymMatrix matrix);
+
 };
 
 #endif
