@@ -49,13 +49,13 @@
 void checkDiscrete(const PContingency &cont, char *measure)
 { if (cont->varType!=TValue::INTVAR)
     if (cont->outerVariable)
-      raiseErrorWho(measure, "cannot evaluate the non-discrete attribute '%s'", cont->outerVariable->name.c_str());
+      raiseErrorWho(measure, "cannot evaluate the non-discrete attribute '%s'", cont->outerVariable->get_name().c_str());
     else
       raiseErrorWho(measure, "cannot evaluate continuous attributes");
 
   if (cont->innerVariable) {
     if (cont->innerVariable->varType != TValue::INTVAR)
-      raiseErrorWho(measure, "cannot work with continuous outcome '%s'", cont->innerVariable->name.c_str());
+      raiseErrorWho(measure, "cannot work with continuous outcome '%s'", cont->innerVariable->get_name().c_str());
   }
   else
     if (!cont->innerDistribution.is_derived_from(TDiscDistribution))
@@ -66,13 +66,13 @@ void checkDiscrete(const PContingency &cont, char *measure)
 void checkDiscreteContinuous(const PContingency &cont, char *measure)
 { if (cont->varType!=TValue::INTVAR)
     if (cont->outerVariable)
-      raiseErrorWho(measure, "cannot evaluate the non-discrete attribute '%s'", cont->outerVariable->name.c_str());
+      raiseErrorWho(measure, "cannot evaluate the non-discrete attribute '%s'", cont->outerVariable->get_name().c_str());
     else
       raiseErrorWho(measure, "cannot evaluate continuous attributes");
 
   if (cont->innerVariable) {
     if (cont->innerVariable->varType != TValue::FLOATVAR)
-      raiseErrorWho(measure, "cannot work with discrete outcome '%s'", cont->innerVariable->name.c_str());
+      raiseErrorWho(measure, "cannot work with discrete outcome '%s'", cont->innerVariable->get_name().c_str());
   }
   else
     if (!cont->innerDistribution.is_derived_from(TContDistribution))
@@ -244,7 +244,7 @@ float TMeasureAttribute::bestThreshold(PDistribution &left_right, float &score,
       score = 0;
       return ILLEGAL_FLOAT;
     }
-    float &score = res.front().second;
+    score = res.front().second;
     float &bestThresh = res.front().first;
     ITERATE(TFloatFloatList, ii, res) {
       if ((*ii).second > score) {
@@ -252,7 +252,7 @@ float TMeasureAttribute::bestThreshold(PDistribution &left_right, float &score,
         score = (*ii).second;
       }
     }
-    return score;
+    return bestThresh;
   }
   
   if (!gen->domain->classVar)
@@ -1116,6 +1116,12 @@ void TMeasureAttribute_relief::prepareNeighbours(PExampleGenerator gen, const in
     for(ei = table.begin(), index = 0; ei; ++ei, index++)
       examplesByClasses.at(int((*ei).getClass())).push_back(index);
 
+    /*
+    This code was removed due to examplesByClasses[neighbourClass].size() later on:
+    since no track is kept on which classes were removed, indexing was misaligned and
+    sometimes caused crashes due to overflow. Instead of this code, classes without examples
+    are skipped (continue in the ITERATE loop below)
+
     for(ebcb = examplesByClasses.begin(), ebci = ebcb, ebce = examplesByClasses.end(); ebci != ebce; ) {
       const int sze = (*ebci).size();
       if (sze)
@@ -1124,7 +1130,7 @@ void TMeasureAttribute_relief::prepareNeighbours(PExampleGenerator gen, const in
         examplesByClasses.erase(ebci);
         ebce = examplesByClasses.end();
       }
-    }
+    }*/
   }
   else {
     ebcb = examplesByClasses.begin(), ebce = examplesByClasses.end();
@@ -1168,6 +1174,8 @@ void TMeasureAttribute_relief::prepareNeighbours(PExampleGenerator gen, const in
 
     ITERATE(vector<vector<int> >, cli, examplesByClasses) {
       const float inCliClass = (*cli).size();
+      if (inCliClass==0)
+          continue;
       vector<pair<int, float> > distances(inCliClass);
       vector<pair<int, float> >::iterator disti = distances.begin(), diste;
       ITERATE(vector<int> , clii, *cli)
