@@ -25,7 +25,10 @@
 #include <string>
 #include <list>
 #include <set>
+#include <map>
+
 #include "orvector.hpp"
+#include "values.hpp"
 
 using namespace std;
 
@@ -51,6 +54,7 @@ WRAPPER(RandomGenerator)
 WRAPPER(Variable);
 WRAPPER(Classifier)
 
+class MMV;
 
 class ORANGE_API TVariable : public TOrange {
 public:
@@ -58,7 +62,8 @@ public:
 
   CLASSCONSTANTS(Type: None=(int)TValue::NONE; Discrete=(int)TValue::INTVAR; Continuous=(int)TValue::FLOATVAR; Other=(int)(TValue::FLOATVAR+1); String=(int)STRINGVAR);
   
-  string name; //P variable's name
+  //string name; //variable's name - hiding it!
+
   int  varType; //P(&Variable_Type) variable type
   bool ordered; //P variable values are ordered
   bool distributed; //P variable values are distributions
@@ -74,26 +79,29 @@ protected:
   TValue DC_value;
   TValue DK_value;
 
+private:
+  string name;
+
+  void registerVariable();
+  void removeVariable();
+
 public:
-  static list<TVariable *> allVariables;
 
-  void *operator new(size_t s) throw(bad_alloc)
-  {
-    void *t = ::operator new(s);
-    TVariable::allVariables.push_back((TVariable *)t);
-    return t;
+  string get_name() const {
+    return name;
   }
 
-  void operator delete(void *t)
-  {
-    /* When the program shuts down, it may happen that the list is destroyed before
-       the variables. We do nothing in this case. */
-    if (allVariables.size())
-      allVariables.remove((TVariable *)t);
-      
-    ::operator delete(t);
+  void set_name(const string &a) {
+    //update the map
+    removeVariable();
+    name = a;
+    registerVariable();
   }
 
+  static MMV allVariablesMap;
+
+  ~TVariable();
+
   /* Status codes for getExisting and make. The codes refer to the difference between
      the requested and the existing variable.
      OK                  the new variable contains at least one of the existing values,
@@ -152,7 +160,7 @@ public:
 
 WRAPPER(Variable)
 
-#define TVarList TOrangeVector<PVariable> 
+#define TVarList TOrangeVector<PVariable>
 VWRAPPER(VarList)
 
 #define TVarListList TOrangeVector<PVarList> 
@@ -226,4 +234,130 @@ public:
   int str2val_low(const string &valname, TValue &valu);
 };
 
+
+#ifdef _MSC_VER
+  #pragma warning(push)
+  #pragma warning(disable: 4275)
+#endif
+
+/* This is to fool pyprops
+#define TValueList _TOrangeVector<float>
+*/
+
+
+
+#define __REGISTER_NO_PYPROPS_CLASS __REGISTER_CLASS
+
+class ORANGE_API TAttributedFloatList : public TOrangeVector<float, false>
+{
+public:
+  __REGISTER_NO_PYPROPS_CLASS
+
+  PVarList attributes;
+
+  inline TAttributedFloatList()
+  {}
+
+  inline TAttributedFloatList(PVarList vlist)
+  : attributes(vlist)
+  {}
+
+  inline TAttributedFloatList(PVarList vlist, const int &i_N, const float &f = 0.0)
+  : TOrangeVector<float, false>(i_N, f),
+    attributes(vlist)
+  {}
+
+  inline TAttributedFloatList(PVarList vlist, const vector<float> &i_X)
+  : TOrangeVector<float,false>(i_X),
+    attributes(vlist)
+  {}
+};
+
+
+class ORANGE_API TAttributedBoolList : public TOrangeVector<bool, false>
+{
+public:
+  __REGISTER_NO_PYPROPS_CLASS
+
+  PVarList attributes;
+
+  inline TAttributedBoolList()
+  {}
+
+  inline TAttributedBoolList(PVarList vlist)
+  : attributes(vlist)
+  {}
+
+  inline TAttributedBoolList(PVarList vlist, const int &i_N, const bool b= false)
+  : TOrangeVector<bool, false>(i_N, b),
+    attributes(vlist)
+  {}
+
+  inline TAttributedBoolList(PVarList vlist, const vector<bool> &i_X)
+  : TOrangeVector<bool, false>(i_X),
+    attributes(vlist)
+  {}
+};
+
+
+
+class ORANGE_API TValueList : public TOrangeVector<TValue, false>
+{
+public:
+  __REGISTER_CLASS
+
+  PVariable variable; //P the variable to which the list applies
+
+  inline TValueList(PVariable var = PVariable())
+  : TOrangeVector<TValue, false>(),
+    variable(var)
+  {}
+ 
+  inline TValueList(const int &N, const TValue &V = TValue(), PVariable var = PVariable())
+  : TOrangeVector<TValue, false>(N, V),
+    variable(var)
+  {}
+
+  inline TValueList(const TOrangeVector<TValue, false> &i_X, PVariable var = PVariable())
+  : TOrangeVector<TValue, false>(i_X),
+    variable(var)
+  {}
+
+  inline TValueList(const TValueList &other)
+  : TOrangeVector<TValue, false>(other),
+    variable(other.variable)
+  {}
+
+  int traverse(visitproc visit, void *arg) const
+  { 
+    TRAVERSE(TOrange::traverse);
+
+    for(TValue *p = _First; p != _Last; p++)
+      if (p->svalV)
+        PVISIT(p->svalV);
+
+    return 0;
+  }
+
+  int dropReferences()
+  { DROPREFERENCES(TOrange::dropReferences);
+    return 0;
+  }
+};
+
+
+WRAPPER(ValueList)
+
+#ifdef _MSC_VER
+  #pragma warning(pop)
+#endif
+
+/* This is to fool pyprops.py
+#define TAttributedFloatList _TOrangeVector<float>
+#define TAttributedBoolList _TOrangeVector<bool>
+*/
+WRAPPER(AttributedFloatList)
+WRAPPER(AttributedBoolList)
+
+
 #endif
